from novadax.kernel.websocket import BaseWebSocketClient


class WebSocketClient:
    def __init__(self, access_key=None, secret_key=None, endpoint='wss://api.novadax.com'):
        self._client = BaseWebSocketClient(endpoint)

    def sub_market_tickers(self, callback):
        self._client.subscribe("MARKET.TICKERS", callback)

    def sub_market_ticker(self, symbol, callback):
        self._client.subscribe(f"MARKET.{symbol}.TICKER", callback)

    def sub_market_depth(self, symbol, callback):
        self._client.subscribe(f"MARKET.{symbol}.DEPTH.LEVEL0", callback)

    def sub_market_trade(self, symbol, callback):
        self._client.subscribe(f"MARKET.{symbol}.TRADE", callback)



import hashlib
import hmac
from time import time
from urllib.parse import urlparse, parse_qsl, quote


class Encryptor:
    _without_content_body_method = ['GET', 'DELETE', 'HEAD']

    def __init__(self, access_key, secret_key):
        self._access_key = access_key
        self._secret_key = secret_key

    def get_access_key(self):
        return self._access_key

    def get_secret_key(self):
        return self._secret_key

    def get_signature(self, method, url, body, timestamp):
        signature_str = self.get_signature_str(method, url, body, timestamp)
        return self.hmac_sha256_hex(self._secret_key, signature_str)

    def get_signature_str(self, method, url, body, timestamp):
        url_parsed = urlparse(url)
        path = url_parsed.path
        params = self.get_sorted_query_params(url_parsed.query)

        if method not in self._without_content_body_method:
            content_md5 = self.md5_hex(body.decode('UTF-8'))
            return '{}\n{}\n{}\n{}'.format(method, path, content_md5, timestamp)
        else:
            return '{}\n{}\n{}\n{}'.format(method, path, params, timestamp)

    @staticmethod
    def get_timestamp():
        return int(time() * 1000)

    @staticmethod
    def get_sorted_query_params(query):
        query_dict = dict(parse_qsl(query))
        sorted_query_item = sorted(query_dict.items(), key=lambda x: x[0], reverse=False)
        sorted_query = map(lambda x: '{}={}'.format(quote(x[0]), quote(x[1])), sorted_query_item)
        return '&'.join(sorted_query)

    @staticmethod
    def hmac_sha256_hex(key, text):
        return hmac.new(bytes(key, 'UTF-8'), bytes(text, 'UTF-8'), hashlib.sha256).hexdigest()

    @staticmethod
    def md5_hex(text):
        obj = hashlib.md5()
        obj.update(bytes(text, 'UTF-8'))
        return obj.hexdigest()



import requests

from novadax.exception import *
from novadax.kernel.http.base_http_auth import BaseHTTPAuth


class BaseHTTPClient:
    def __init__(self, endpoint, access_key=None, secret_key=None):
        self._endpoint = endpoint
        if access_key and secret_key:
            self._auth = BaseHTTPAuth(access_key, secret_key)
        else:
            self._auth = None

    def get(self, path, params=None):
        return self._request(False, 'get', path, params=params)

    def post(self, path, params=None, json=None):
        return self._request(False, 'post', path, params=params, json=json)

    def get_with_auth(self, path, params=None):
        return self._request(True, 'get', path, params=params)

    def post_with_auth(self, path, params=None, json=None):
        return self._request(True, 'post', path, params=params, json=json)

    def _request(self, auth, method, path, **kwargs):
        url = self._endpoint + path
        if auth and not self._auth:
            raise RuntimeException('A99999', 'access key and secret key is required.')
        elif auth:
            kwargs['auth'] = self._auth

        if 'params' in kwargs:
            if kwargs['params'] is None:
                del kwargs['params']
            else:
                kwargs['params'] = self._filter_none(kwargs['params'])

        if 'json' in kwargs:
            if kwargs['json'] is None:
                del kwargs['json']
            else:
                kwargs['json'] = self._filter_none(kwargs['json'])

        if 'timeout' not in kwargs:
            kwargs['timeout'] = 30

        try:
            r = requests.request(method, url, **kwargs)
        except requests.exceptions.RequestException as err:
            raise RuntimeException('A99999', str(err))

        result = r.json()

        if r.status_code >= 400:
            raise RequestException(result['code'], result['message'])
        return result

    @staticmethod
    def _filter_none(data):
        if isinstance(data, dict):
            return {k: v for k, v in data.items() if v is not None}
        return data



from requests.auth import AuthBase

from novadax.kernel.encryptor import Encryptor


class BaseHTTPAuth(AuthBase, Encryptor):
    _without_content_body_method = ['GET', 'DELETE', 'HEAD']

    def __init__(self, access_key, secret_key):
        Encryptor.__init__(self, access_key, secret_key)

    def __call__(self, r):
        timestamp = self.get_timestamp()
        r.headers['X-Nova-Access-Key'] = self._access_key
        r.headers['X-Nova-Signature'] = self.get_signature(r.method, r.url, r.body, timestamp)
        r.headers['X-Nova-Timestamp'] = timestamp
        return r



from novadax.kernel import BaseHTTPClient


class RequestClient:
    def __init__(self, access_key=None, secret_key=None, endpoint='https://api.novadax.com'):
        self._client = BaseHTTPClient(endpoint, access_key, secret_key)

    def get_timestamp(self):
        return self._client.get('/v1/common/timestamp')

    def get_symbol(self, symbol):
        return self._client.get('/v1/common/symbol', {
            'symbol': symbol
        })

    def list_symbols(self):
        return self._client.get('/v1/common/symbols')

    def list_tickers(self):
        return self._client.get('/v1/market/tickers')

    def get_ticker(self, symbol):
        return self._client.get('/v1/market/ticker', {
            'symbol': symbol
        })

    def get_depth(self, symbol, limit=100):
        return self._client.get('/v1/market/depth', {
            'symbol': symbol,
            'limit': limit
        })

    def list_trades(self, symbol, limit=100):
        return self._client.get('/v1/market/trades', {
            'symbol': symbol,
            'limit': limit
        })

    def get_kline(self, symbol, unit, from_score, to_score):
        return self._client.get('/v1/market/kline/history', {
            'symbol': symbol,
            'unit': unit,
            'from': from_score,
            'to': to_score
        })

    def get_order(self, _id):
        return self._client.get_with_auth('/v1/orders/get', {
            'id': _id
        })

    def list_orders(self, symbol, status=None,
                    from_id=None, to_id=None,
                    from_timestamp=None, to_timestamp=None,
                    account_id=None, limit=100):
        return self._client.get_with_auth('/v1/orders/list', {
            'accountId': account_id,
            'symbol': symbol,
            'status': status,
            'fromId': from_id,
            'toId': to_id,
            'fromTimestamp': from_timestamp,
            'toTimestamp': to_timestamp,
            'limit': limit
        })

    def create_order(self, symbol, _type, side, price=None, amount=None, value=None, account_id=None):
        return self._client.post_with_auth('/v1/orders/create', {}, {
            'accountId': account_id,
            'symbol': symbol,
            'type': _type,
            'side': side,
            'price': price,
            'amount': amount,
            'value': value
        })

    def cancle_order(self, _id):
        return self._client.post_with_auth('/v1/orders/cancel', {}, {
            'id': _id
        })

    def list_order_fills(self, order_id=None, symbol=None,
                         from_id=None, to_id=None,
                         from_timestamp=None, to_timestamp=None,
                         account_id=None, limit=100):
        return self._client.get_with_auth('/v1/orders/fills', {
            'accountId': account_id,
            'orderId': order_id,
            'symbol': symbol,
            'fromId': from_id,
            'toId': to_id,
            'fromTimestamp': from_timestamp,
            'toTimestamp': to_timestamp,
            'limit': limit
        })

    def get_account_balance(self):
        return self._client.get_with_auth('/v1/account/getBalance')

    def get_account_balance_current(self):
        return self._client.get_with_auth('/v1/account/getBalance/current')

    def withdraw_coin(self, code, amount, toAddr, tag=None):
        return self._client.post_with_auth('/v1/account/withdraw/coin', {}, {
            'amount': amount,
            'code': code,
            'wallet': toAddr,
            'tag': tag
        })

    def subs(self):
        return self._client.get_with_auth('/v1/account/subs')

    def subs_balance(self, sub_id):
        return self._client.get_with_auth('/v1/account/subs/balance', {
            "subId": sub_id
        })

    def subs_transfer(self, sub_id, currency, transfer_amount, transfer_type):
        return self._client.post_with_auth('/v1/account/subs/transfer', {}, {
            'subId': sub_id,
            'currency': currency,
            'transferAmount': transfer_amount,
            'transferType': transfer_type
        })

    def subs_transfer_record(self, sub_id):
        return self._client.get_with_auth('/v1/account/subs/transfer/record', {
            'subId': sub_id
        })



class NovadaxException(Exception):
    def __init__(self, code, message):
        self.code = code
        self.message = message


class RuntimeException(NovadaxException):
    pass


class RequestException(NovadaxException):
    pass



from setuptools import setup, find_packages

setup(
    name="novadax",
    version="1.1.2",
    author="NovaDAX",
    author_email="support@novadax.com",
    description="NovaDAX API SDK",
    url="https://doc.novadax.com/",
    packages=find_packages(exclude=["tests*"]),
    install_requires=[
        'requests>=2,<3',
        'python-socketio[client]'
    ]
)



import socketio


class BaseWebSocketClient:

    def __init__(self, endpoint):
        self._endpoint = endpoint
        self._socket = None

    def get_socket(self):
        if self._socket is None:
            print(self._endpoint)
            self._socket = socketio.Client()
            self._socket.connect(self._endpoint, transports=['websocket'])
        return self._socket

    def subscribe(self, topic, callback):
        self.get_socket().emit("SUBSCRIBE", [topic])
        self.get_socket().on(topic, callback)

    def unsubscribe(self, topic):
        self.get_socket().emit("UNSUBSCRIBE", [topic])
